#include "GLWindow.h"
#include <iostream>
#include "ngl/Camera.h"
#include "ngl/Colour.h"
#include "ngl/Light.h"
#include "ngl/Matrix.h"
#include "ngl/Transformation.h"
#include "ngl/TransformStack.h"
#include "ngl/Material.h"
#include "ngl/NGLInit.h"
#include "ngl/Obj.h"
#include "ngl/VBOPrimitives.h"
#include "ngl/ShaderManager.h"

//----------------------------------------------------------------------------------------------------------------------
GLWindow::GLWindow(
                   QWidget *_parent
                  ) :
                    QGLWidget(_parent)
{

  // set this widget to have the initial keyboard focus
  setFocus();
  // re-size the widget to that of the parent (in this case the GLFrame passed in on construction)
  this->resize(_parent->size());
  // Now set the initial GLWindow attributes to default values
  // Roate is false
  m_rotate=false;
  // mouse rotation values set to 0
  m_spinXFace=0;
  m_spinYFace=0;
  m_fps=0;
  m_frames=0;
	m_animate=true;

}

// This virtual function is called once before the first call to paintGL() or resizeGL(),
//and then once whenever the widget has been assigned a new QGLContext.
// This function should set up any required OpenGL context rendering flags, defining display lists, etc.

//----------------------------------------------------------------------------------------------------------------------
void GLWindow::initializeGL()
{

  glClearColor(0.0f, 0.0f, 0.0f, 1.0f);			   // Grey Background
  ngl::NGLInit *Init = ngl::NGLInit::instance();
  #ifdef WIN32
  	glewInit();
  #endif
  Init->initGlew();
  Init->initVBO();


	 m_cam = new ngl::PathCamera(ngl::Vector(0,1,-0.2),"data/loop.dat",0.001,ngl::PERSPECTIVE);
	 m_cam->setShape(40.0,1024.0/720.0,0.5,150.0,ngl::PERSPECTIVE);
	 m_cam->updateLooped();
   // grab an instance of shader manager
   ngl::ShaderManager *shader=ngl::ShaderManager::instance();
   // load a frag and vert shaders
   shader->loadShader("PointSprite","shaders/Vertex.vs","shaders/Fragment.fs");
   // set this as the active shader
   shader->useShader("PointSprite");
   // now pass the modelView and projection values to the shader
   shader->setShaderParamFromMatrix("PointSprite","ViewMatrix",m_cam->getModelView());
   shader->setShaderParamFromMatrix("PointSprite","projectionMatrix",m_cam->getProjection());

   shader->loadShader("Colour","shaders/Colour.vs","shaders/Colour.fs");
   // set this as the active shader
   shader->useShader("Colour");
   // now pass the modelView and projection values to the shader
   shader->setShaderParamFromMatrix("Colour","ViewMatrix",m_cam->getModelView());
   shader->setShaderParamFromMatrix("Colour","projectionMatrix",m_cam->getProjection());
   // the shader will use the currently active material and light0 so set them
	 shader->setShaderParam4f("Colour","Colour",0.2,0.2,0.2,1);
	 glEnable(GL_BLEND);
	 glBlendFunc(GL_ONE, GL_ONE);
	 glEnable(GL_PROGRAM_POINT_SIZE_EXT);
	 glEnable(GL_POINT_SPRITE);

   glPointParameterfARB( GL_POINT_FADE_THRESHOLD_SIZE_ARB, 32.0f );
	 glPointParameteri(GL_POINT_SPRITE_COORD_ORIGIN,GL_LOWER_LEFT);
	 glPointParameterfARB( GL_POINT_SIZE_MIN_ARB, 1.0f );
	 glPointParameterfARB( GL_POINT_SIZE_MAX_ARB, 32.0f );


	 m_emitter1 = new Emitter(ngl::Vector(0,0,0),"textures/SmokeTest2.tiff",2000);
	 m_emitter2 = new Emitter(ngl::Vector(2,0,2),"textures/SmokeTest.tiff",4000);
	 m_emitter3 = new Emitter(ngl::Vector(0,0,0),"textures/SmokeTest3.tiff",4000);

	 for(int i=0; i<500; ++i)
	 {
		 m_emitter1->update();
		 m_emitter2->update();
		 m_emitter3->update();
	 }
	 ngl::VBOPrimitives *prim=ngl::VBOPrimitives::instance();
	 prim->createVBOQuadPlane("plane",10,10,40,40,ngl::Vector(0,1,0));
	 m_fpsTimer =startTimer(10);
	 m_camTimer=startTimer(20);

}

//----------------------------------------------------------------------------------------------------------------------
//This virtual function is called whenever the widget has been resized.
// The new size is passed in width and height.
void GLWindow::resizeGL(
                        int _w,
                        int _h
                       )
{
  glViewport(0,0,_w,_h);
  m_cam->setShape(65,(float)_w/_h,0.5,150,ngl::PERSPECTIVE);
  ngl::ShaderManager *shader=ngl::ShaderManager::instance();
	(*shader)["PointSprite"]->use();
  shader->setShaderParamFromMatrix("PointSprite","projectionMatrix",m_cam->getProjection());
	(*shader)["Colour"]->use();
  shader->setShaderParamFromMatrix("Colour","projectionMatrix",m_cam->getProjection());

}

//----------------------------------------------------------------------------------------------------------------------
//This virtual function is called whenever the widget needs to be painted.
// this is our main drawing routine
void GLWindow::paintGL()
{
  // grab an instance of the shader manager
  ngl::ShaderManager *shader=ngl::ShaderManager::instance();
  (*shader)["PointSprite"]->use();
  // clear the screen and depth buffer
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  // Rotation based on the mouse position for our global
  // transform
  ngl::Transformation trans;
  trans.setRotation(m_spinXFace,m_spinYFace,0);
  // set this in the TX stack
  m_transformStack.setGlobal(trans);
  // now set this value in the shader for the current ModelMatrix
  shader->setShaderParamFromMatrix("PointSprite","ViewMatrix",m_cam->getModelView());

  shader->setShaderParamFromMatrix("PointSprite","ModelMatrix",m_transformStack.getCurrAndGlobal().getMatrix());
  // get the VBO instance and draw the built in teapot

  glBindTexture(GL_TEXTURE_2D,m_emitter1->getTextureID());
	glTexEnvf(GL_POINT_SPRITE, GL_COORD_REPLACE, GL_TRUE);

	m_emitter1->draw();

  glBindTexture(GL_TEXTURE_2D,m_emitter2->getTextureID());
	glTexEnvf(GL_POINT_SPRITE, GL_COORD_REPLACE, GL_TRUE);

	m_emitter2->draw();

  glBindTexture(GL_TEXTURE_2D,m_emitter3->getTextureID());
	glTexEnvf(GL_POINT_SPRITE, GL_COORD_REPLACE, GL_TRUE);

	m_emitter3->draw();


  (*shader)["Colour"]->use();
  shader->setShaderParamFromMatrix("Colour","ViewMatrix",m_cam->getModelView());
  shader->setShaderParamFromMatrix("Colour","ModelMatrix",m_transformStack.getCurrAndGlobal().getMatrix());

	glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
	ngl::VBOPrimitives *prim=ngl::VBOPrimitives::instance();
	prim->draw("plane");


}

//----------------------------------------------------------------------------------------------------------------------
void GLWindow::mouseMoveEvent (
                               QMouseEvent * _event
                              )
{
  // note the method buttons() is the button state when event was called
  // this is different from button() which is used to check which button was
  // pressed when the mousePress/Release event is generated
  if(m_rotate && _event->buttons() == Qt::LeftButton)
  {
    m_spinYFace = ( m_spinYFace + (_event->x() - m_origX) ) % 360 ;
    m_spinXFace = ( m_spinXFace + (_event->y() - m_origY) ) % 360 ;
    m_origX = _event->x();
    m_origY = _event->y();
  }
  // re-draw GL
  updateGL();
}


//----------------------------------------------------------------------------------------------------------------------
void GLWindow::mousePressEvent (
                                QMouseEvent * _event
                               )
{
  // this method is called when the mouse button is pressed in this case we
  // store the value where the maouse was clicked (x,y) and set the Rotate flag to true
  if(_event->button() == Qt::LeftButton)
  {
    m_origX = _event->x();
    m_origY = _event->y();
    m_rotate =true;
  }
}

//----------------------------------------------------------------------------------------------------------------------
void GLWindow::mouseReleaseEvent (
                                  QMouseEvent * _event
                                 )
{
  // this event is called when the mouse button is released
  // we then set Rotate to false
  if (_event->button() == Qt::LeftButton)
  {
    m_rotate=false;
  }
}

void GLWindow::timerEvent(
                          QTimerEvent *_event
                         )
{

	if( m_animate == true)
	{
		m_emitter1->update();
		m_emitter2->update();
		m_emitter3->update();

	}
	if(_event->timerId() == m_camTimer)
	{
		m_cam->updateLooped();
	}

	// re-draw GL
	updateGL();
}
