#include "GLWindow.h"
#include "ngl/Light.h"
#include "ngl/TransformStack.h"
#include "ngl/Material.h"
#include "ngl/NGLInit.h"
#include "ngl/ShaderManager.h"

//----------------------------------------------------------------------------------------------------------------------
GLWindow::GLWindow(
                   QWidget *_parent
                  ) :
                    QGLWidget(_parent)
{

  // set this widget to have the initial keyboard focus
  setFocus();
  // re-size the widget to that of the parent (in this case the GLFrame passed in on construction)
  this->resize(_parent->size());

	m_particleTimer=startTimer(20);

}

// This virtual function is called once before the first call to paintGL() or resizeGL(),
//and then once whenever the widget has been assigned a new QGLContext.
// This function should set up any required OpenGL context rendering flags, defining display lists, etc.

//----------------------------------------------------------------------------------------------------------------------
void GLWindow::initializeGL()
{

  glClearColor(0.4f, 0.4f, 0.4f, 1.0f);			   // Grey Background
  // enable depth testing for drawing
   glEnable(GL_DEPTH_TEST);
   // Now we will create a basic Camera from the graphics library
   // This is a static camera so it only needs to be set once
   // First create Values for the camera position
   ngl::Vector From(0,5,18);
   ngl::Vector To(0,0,0);
   ngl::Vector Up(0,1,0);
   ngl::NGLInit *Init = ngl::NGLInit::instance();
   #ifdef WIN32
   	glewInit();
   #endif
   Init->initGlew();
   Init->initVBO();
   m_cam= new ngl::Camera(From,To,Up,ngl::PERSPECTIVE);
   // set the shape using FOV 45 Aspect Ratio based on Width and Height
   // The final two are near and far clipping planes of 0.5 and 10
   m_cam->setShape(40,(float)720.0/576.0,0.5,150,ngl::PERSPECTIVE);
   // now to load the shader and set the values
   // grab an instance of shader manager
   ngl::ShaderManager *shader=ngl::ShaderManager::instance();
   // load a frag and vert shaders
   shader->loadShader("Blinn","shaders/Vertex.vs","shaders/Fragment.fs");
   // set this as the active shader
   shader->useShader("Blinn");
   // now pass the modelView and projection values to the shader
   shader->setShaderParamFromMatrix("Blinn","ViewMatrix",m_cam->getModelView());
   shader->setShaderParamFromMatrix("Blinn","projectionMatrix",m_cam->getProjection());
   // the shader will use the currently active material and light0 so set them
   ngl::Material m(ngl::GOLD);
   m.use();
   ngl::Light *L1 = new ngl::Light(ngl::Vector(5,12,0,1),ngl::Colour(1,1,1,1),ngl::LIGHTLOCAL);
   L1->enable();
   ngl::VBOPrimitives *prim=ngl::VBOPrimitives::instance();

   prim->createVBOSphere("sphere",0.1,10);
	 m_wind=new ngl::Vector(1,1,1);
	 m_emitter = new Emitter(ngl::Vector(0,0,0),400,m_wind);


}

//----------------------------------------------------------------------------------------------------------------------
//This virtual function is called whenever the widget has been resized.
// The new size is passed in width and height.
void GLWindow::resizeGL(
                        int _w,
                        int _h
                       )
{
  glViewport(0,0,_w,_h);
  m_cam->setShape(65,(float)_w/_h,0.5,150,ngl::PERSPECTIVE);
  ngl::ShaderManager *shader=ngl::ShaderManager::instance();
  shader->setShaderParamFromMatrix("Blinn","projectionMatrix",m_cam->getProjection());
}

//----------------------------------------------------------------------------------------------------------------------
//This virtual function is called whenever the widget needs to be painted.
// this is our main drawing routine
void GLWindow::paintGL()
{
  // grab an instance of the shader manager
  ngl::ShaderManager *shader=ngl::ShaderManager::instance();
  shader->useShader("Blinn");
  // clear the screen and depth buffer
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  m_emitter->draw();

	glUseProgramObjectARB(0);
  glColor3f(1,1,1);
  QFont font;
  font.setPointSize(20);
  QString text=QString("Wind Vector  %1 %2 %3").arg(m_wind->m_x).arg(m_wind->m_y).arg(m_wind->m_z);
  renderText(10,20,text,font);
}

void GLWindow::keyPress(
												QKeyEvent *_event
											 )
{
  // this method is called every time the main window recives a key event.
  // we then switch on the key value and set the camera in the GLWindow
  switch (_event->key())
  {
		case Qt::Key_Up : m_wind->m_y+=0.1; break;
		case Qt::Key_Down : m_wind->m_y-=0.1; break;
		case Qt::Key_Left : m_wind->m_x+=0.1; break;
		case Qt::Key_Right : m_wind->m_x-=0.1; break;

		case Qt::Key_I : m_wind->m_z+=0.1; break;
		case Qt::Key_O : m_wind->m_z-=0.1; break;

		case Qt::Key_Space : m_wind->set(1,1,1); break;
	}
	updateGL();

}
void GLWindow::timerEvent(
                          QTimerEvent *_event
                         )
{
	if(_event->timerId() ==   m_particleTimer)
	{
		m_emitter->update();
		updateGL();
	}
    // re-draw GL
}
